@@ -39,3 +39,17 @@ convert_range = 0
 # lookup on every call. 
 # If this is 0 it simply creates a wrapper. 
 lookup_module_cpdef = 0
+
+# This will set local variables to None rather than NULL which may cause 
+# surpress what would be an UnboundLocalError in pure Python but eliminates 
+# checking for NULL on every use, and can decref rather than xdecref at the end. 
+# WARNING: This is a work in progress, may currently segfault.
+init_local_none = 1
+
+# Optimize no argument and one argument methods by using the METH_O and METH_NOARGS
+# calling conventions. These are faster calling conventions, but disallow the use of 
+# keywords (which, admittedly, are of little use in these cases). 
+optimize_simple_methods = 1
+
+# Append the c file and line number to the traceback for exceptions. 
+c_line_in_traceback = 0
